import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;

/**
 * Matter is a superclass for everything that has gravity.
 * If you want the class to be affected by gravity, put the line gravitate();
 * in the act method.
 * 
 * A reasonable number for mass is about 100 - 5000 depending on the scale.
 * 
 * @author David Osborne
 * @version 0.3
 * @date 18/10/2008
 */
public abstract class Matter extends SmoothMover
{
    // If the mass is 0, the equations mess up
    double mass = 0.1;
    double speedLimit = 4;
    
    public Matter(){}
    public Matter(Vector v){
        super(v);
    }
    
    public double getMass(){
        return mass;
    }
    
    public void moveNicely(){
        move();
        limitSpeed();
        stayIn();
    }
    
    /**
     * Prevents the object from going faster than the speedLimit
     */
    public void limitSpeed(){
        double velocity = getSpeed().getLength();
        if (velocity > speedLimit){
            double adjustment = speedLimit - velocity;
            increaseSpeed(new Vector(getSpeed().getDirection(), adjustment));
        }
    }
    
    
    /**
     * Bounces off walls
     */
    public void stayIn(){
        Vector speed = getSpeed();
        if(getX() >= getWorld().getWidth()-1 || getX() <= 0 ) {
            int newAngle = 540-speed.getDirection();
            Vector v = DirectionUtil.toVector(speed.getX()*(-2), speed.getY());
            increaseSpeed(v);
        }
        
        if(getY() >= getWorld().getHeight()-1 || getY() <= 0) {
           int newAngle = 360-speed.getDirection();
           Vector v = DirectionUtil.toVector(speed.getX(), speed.getY()*(-2));
           increaseSpeed(v);
        }
    }
    
    /**
     * Iterates through a list of all the Matter objects in the world
     * and adds the appropriate acceleration.
     */
    public void gravitate(){

        List bodies = getWorld().getObjects(Matter.class);
        for (Object body : bodies){
            if (body != this){
            increaseSpeed(calculateGravity((Matter) body));
        }
        }
    }

    
    /**
     * Returns the distance and angle (in degrees) as a vector
     * from this object to another actor in the world
     */
    public Vector getBearing(Actor obj){
        int objX = obj.getX();
        int objY = obj.getY();
        
        int x = objX - getX();
        int y = objY - getY();
        double theta = Math.atan2(y, x);
        int direction = (int) (Math.toDegrees(theta));
        double radius = (Math.hypot(x, y));
        Vector v = new Vector(direction, radius);
        return v;
    }
    
    /**
     * Calculates gravitational force when F = (m1xm2)/r^2
     * Then, uses that force to calculate the acceleration when F = Ma
     * Returns a vector containing the acceleration 
     * and the direction (in degrees) of the other object
     */
    public Vector calculateGravity(Matter obj){
        
        Vector bearing = getBearing(obj);
        double force = (mass * obj.mass)/Math.pow(bearing.getLength(), 2);
        double acceleration = force/mass;
        return new Vector(bearing.getDirection(), acceleration);

    }
    
    /**
     * Returns true if the object has collided with another object of class cls.
     */
    public boolean hasCollided(Class cls){
        Object object = getOneIntersectingObject(cls);
        if (object != null && object != this){
            return true;
        }
        else{
            return false;
        }
    }
}
